@@ -2,28 +2,22 @@
 # -*- coding: utf-8 -*-
 import os
 import sys
+import subprocess
 
 
 def main(cli_args=None):
     if not cli_args:
         cli_args = sys.argv[1:]
-    cmd = os.path.abspath(
-        os.path.join(
-            os.path.dirname(__file__), '%s.sh' % os.path.basename(__file__)[0:-3]
-        )
-    )
+    cmd = '%s.sh' % os.path.splitext(os.path.abspath(__file__))[0]
     if not os.path.isfile(cmd):
-        cmd = os.path.abspath(
-            os.path.join(
-                os.path.dirname(__file__),
-                '..',
-                '%s.sh' % os.path.basename(__file__)[0:-3]
-            )
-        )
+        cmd = os.path.split(cmd)
+        cmd = os.path.join(os.path.dirname(cmd[0]), cmd[1])
     if not os.path.isfile(cmd):
         print('Internal package error: file %s not found!' % cmd)
     for arg in cli_args:
-        if ' ' in arg:
+        if '<' in arg or '>' in arg:
+            arg = "'%s'" % arg.replace("'", r"\'")
+        elif ' ' in arg:
             if '"' in arg:
                 arg = '"%s"' % arg.replace('"', r'\"')
             else:
@@ -35,4 +29,8 @@ def main(cli_args=None):
         else:
             arg = '%s' % arg
         cmd = '%s %s' % (cmd, arg)
-    return os.system(cmd)
+    return subprocess.call(cmd, shell=True)
+
+
+if __name__ == "__main__":
+    exit(main())
